#pragma once

#include <unordered_map>
#include <vector>
#include <string>
#include <iostream>
#include <exception>

struct BBBParseError: std::exception {
	BBBParseError(std::string msg): message(msg) {}
	std::string message;

	const char *what() const noexcept override {
		return message.c_str();
	}
};

class BBBParser {
public:
	using Variables = std::unordered_map<std::string, std::vector<std::string>>;

	static const int FLAG_NONE = 0;
	static const int FLAG_INSECURE = 1 << 0;

	BBBParser(std::istream &stream, int flags, int line = 1, int ch = 1):
		flags_(flags), line_(line), ch_(ch), stream_(stream) {}

	void parseLine(const Variables &vars, std::vector<std::string> &values);
	void parse(Variables &vars);

	int line_;
	int ch_;
	int flags_;

private:
	int peek() { return stream_.peek(); }
	int get();
	void skip(char);
	void skip() { get(); }
	void error(std::string);

	void skipWhitespaceLine();
	void skipWhitespace();

	char parseEscape();
	void parseExpansion(const Variables &vars, std::vector<std::string> &values);
	void parseQuotedExpansion(const Variables &vars, std::string &content);
	void parseQuotedString(const Variables &vars, std::string &content);
	bool parseString(const Variables &vars, std::string &content, int sep = -1);

	std::istream &stream_;
};
